import struct
from .producers import grpc_producer
from ..http.request import XMLRPCRequest

class GRPCRequest (XMLRPCRequest):
	initial_http_version = "2.0"
	
	def __init__ (self, uri, method, params = (), headers = None, auth = None, logger = None, meta = {}):
		self.uri = uri
		self.method = method
		self.params = params
		self.auth = (auth and type (auth) is not tuple and tuple (auth.split (":", 1)) or auth)
		self.logger = logger
		self.meta = meta
		self.address, self.path = self.split (uri)
	
		self.headers = {
			"grpc-timeout": "10S", 
			"grpc-encoding": "gzip",
			"grpc-accept-encoding": "identity,gzip",
			"user-agent": self.user_agent,
			"message-type": self.params [0].__class__.__name__,
		}		
		self.payload = self.serialize ()
		if not self.payload:
			self.method = "GET"
		else:
			self.headers ["Content-Type"] = "application/grpc"		
		
	def get_cache_key (self):
		return None
		
	def xmlrpc_serialized (self):
		return False
			
	def split (self, uri):
		(host, port), path = XMLRPCRequest.split (self, uri)				
		if path [-1] != "/":
			path += "/"
			self.uri += "/"
		path += self.method
		self.uri += self.method
		return (host, port), path
	
	def serialize (self):		
		return grpc_producer (self.params [0])
	


import os

if os.name == "nt":
	import subprocess
	def kill (pid):	
		subprocess.call(['taskkill', '/F', '/T', '/PID', str(pid)])		

else:
	import psutil
	
	def kill (pid, including_parent = True):	
		parent = psutil.Process(pid)
		children = parent.children(recursive=True)
		for child in children:
			child.kill()
		psutil.wait_procs(children, timeout=5)
		if including_parent:
			parent.kill()
			parent.wait(5)



import base64
from hashlib import md5
import os
from . import http_util

class Authorizer:
	def __init__ (self):
		self.db = {}
	
	def get (self, netloc, auth, method, uri, data):
		if netloc not in self.db:
			return ""
			
		infod = self.db [netloc]
		if infod ["meth"] == "basic":
			return "Basic " + base64.encodestring ("%s:%s" % auth) [:-1]
		elif infod ["meth"] == "bearer":
			return "Bearer " + auth [0]
		else:
			infod ["nc"] += 1
			hexnc = hex (infod ["nc"])[2:].zfill (8)
			infod ["cnonce"] = http_util.md5uniqid ()
			
			A1 = md5 (("%s:%s:%s" % (auth [0], infod ["realm"], auth [1])).encode ("utf8")).hexdigest ()
			if infod ["qop"] == "auth":
				A2 = md5 (("%s:%s" % (method, uri)).encode ("utf8")).hexdigest ()
			elif infod ["qop"] == "auth-int" and type (data) is bytes:
				entity = md5 (data).hexdigest ()
				A2 = md5 (("%s:%s:%s" % (method, uri, entity)).encode ("utf8")).hexdigest ()
			else:
				return # sorry data is not bytes or unknown qop
						
			Hash = md5 (("%s:%s:%s:%s:%s:%s" % (
				A1,
				infod ["nonce"],
				hexnc,
				infod ["cnonce"],
				infod ["qop"],
				A2
				)).encode ("utf8")
			).hexdigest ()
			
			return (
				'Digest username="%s", realm="%s", nonce="%s", '
				'uri="%s", response="%s", opaque="%s", qop=%s, nc=%s, cnonce="%s"' % (
					auth [0], infod ["realm"], infod ["nonce"], uri, Hash, 
					infod ["opaque"], infod ["qop"], hexnc, infod ["cnonce"]
				)
			)
			
	def set (self, netloc, authreq, auth):
		if auth is None:
			return
		
		amethod, authinfo = authreq.split (" ", 1)		
		infod = {"meth": amethod.lower ()}
		infod ["nc"] = 0
		for info in authinfo.split (","):
			k, v = info.strip ().split ("=", 1)
			if not v: return self.get_www_authenticate ()
			if v[0] == '"': v = v [1:-1]
			infod [k]	 = v
		
		if "qop" in infod:
			qop = list (map (lambda x: x.strip (), infod ["qop"].split (",")))
			if "auth" in qop:
				infod ["qop"] = "auth"
			else:
				infod ["qop"] = "auth-int"
				
		self.db [netloc] = infod
		
	def make_http_auth_header (self, request, is_proxy = False):
		auth = request.get_auth ()
		if auth:
			uri = is_proxy and request.uri or request.path
			auth_header = self.get (request.get_address (), auth, request.get_method (), uri, request.get_payload ())
			if auth_header is None:
				raise AssertionError ("Unknown authedentification method")
			return auth_header			
	
	def save_http_auth_header (self, request, response):
		self.set (
			request.get_address (), 
			response.get_header ("WWW-Authenticate"), 
			request.get_auth ()
		)	

authorizer = Authorizer ()




import subprocess
import os

def get_id():
    if 'nt' in os.name:
        return subprocess.Popen('dmidecode.exe -s system-uuid'.split())
    else:
        return subprocess.Popen('hal-get-property --udi /org/freedesktop/Hal/devices/computer --key system.hardware.uuid'.split())
		
		
if __name__ == "__main__":
	print (get_id())
	


import requests
import json
from urllib.parse import quote

def tostr (p):
	if p:
		return "?" + "&".join (["%s=%s" % (k, quote (str (v))) for k, v in p.items ()])	
	return ""
			
class API:
	API_SERVER = "http://127.0.0.1:5000"
	REQ_TIMEOUT = 30.0
	
	def __init__ (self, server, timeout = 30.0):
		self.API_SERVER = server
		self.REQ_TIMEOUT = timeout
		self.session = requests.Session ()
			
	def post (self, uri, d):
		return self.session.post ("%s/%s" % (self.API_SERVER, uri), json.dumps (d), timeout = self.REQ_TIMEOUT).json ()
	
	def put (self, uri, d):
		return self.session.put ("%s/%s" % (self.API_SERVER, uri), json.dumps (d), timeout = self.REQ_TIMEOUT).json ()
	
	def get (self, uri, p = {}):	
		return self.session.get ("%s/%s%s" % (self.API_SERVER, uri, tostr (p)), timeout = self.REQ_TIMEOUT).json ()
	
	def delete (self, uri, p = {}):
		return self.session.delete ("%s/%s%s" % (self.API_SERVER, uri, tostr (p)), timeout = self.REQ_TIMEOUT).json ()





class AttrDict (dict):
	def __init__(self, *args, **kwargs):
		super(AttrDict, self).__init__(*args, **kwargs)
		self.__dict__ = self
	

class CaseInsensitiveKey(object):
	def __init__(self, key):
		self.key = key
		
	def __hash__(self):
		return hash(self.key.lower())
	
	def __eq__(self, other):
		return self.key.lower() == other.key.lower()
	
	def __str__(self):
		return self.key
	
	def __repr__(self):
		return self.key	
	
	def __getattr__ (self, name):
		return getattr (self.key, name)
		

class NocaseDict(dict):
	def __setitem__(self, key, value):
		key = CaseInsensitiveKey(key)
		super(NocaseDict, self).__setitem__(key, value)
	
	def __getitem__(self, key):
		key = CaseInsensitiveKey(key)
		return super(NocaseDict, self).__getitem__(key)
		

if __name__ == "__main__":
	a = AttrDict ()
	a ["a-s"] = 1
	print (a.a_s)
	a.x = 4
	print (a['x'])
	
	b = NocaseDict ()
	b ['Content-Length'] = 100
	print (b ['content-length'])


"""
Hans Roh 2015 -- http://osp.skitai.com
License: BSD
"""
import re
import sys
import os
import shutil, glob
from warnings import warn
try:
	from setuptools import setup
except ImportError:
	from distutils.core import setup

if sys.argv[-1] == 'publish':
	if os.name == "nt":
		os.system('python setup.py sdist upload') # bdist_wininst --target-version=2.7
	else:
		os.system('python setup.py sdist upload')
	sys.exit()

classifiers = [
  'License :: OSI Approved :: MIT License',
  'Development Status :: 3 - Alpha',
  'Topic :: Internet :: WWW/HTTP',
  'Topic :: Database :: Front-Ends',
	'Environment :: Console',	
	'Topic :: Software Development :: Libraries :: Python Modules',
	'Intended Audience :: Developers',
	'Intended Audience :: Science/Research',
	'Programming Language :: Python',	
	'Programming Language :: Python :: 3'
]

PY_MAJOR_VERSION = sys.version_info [0]
if PY_MAJOR_VERSION == 3:
	if os.path.isfile ("aquests/lib/py2utils.py"):
		os.remove ("aquests/lib/py2utils.py")
		try: os.remove ("aquests/lib/py2utils.pyc")
		except OSError: pass		
else:
	if not os.path.isfile ("aquests/lib/py2utils.py"):
		with open ("aquests/lib/py2utils.py", "w") as f:
			f.write ("def reraise(type, value, tb):\n\traise type, value, tb\n")			

packages = [
	'aquests',
	'aquests.client',
	'aquests.dbapi',	
	'aquests.protocols',
	'aquests.protocols.dns',
	'aquests.protocols.dns.pydns',
	'aquests.protocols.http',	
	'aquests.protocols.http2',
	'aquests.protocols.ws',	
	'aquests.protocols.smtp',
	'aquests.protocols.grpc',
	'aquests.protocols.proxy',
	'aquests.lib',
	'aquests.lib.athreads',
	'aquests.lib.nets'
]

package_dir = {'aquests': 'aquests'}

package_data = {
	"aquests": [
		"protocols/dns/*.txt",
		"protocols/dns/pydns/*.txt"	
	]
}

install_requires = [
	"redis==2.10", "pymongo==3.4.0", 
	"h2==2.5.1", "protobuf==3.1.0.post1", 
	"psycopg2==2.6.2"
]

with open('aquests/__init__.py', 'r') as fd:
	version = re.search(r'^__version__\s*=\s*"(.*?)"',fd.read(), re.M).group(1)

setup(
	name='aquests',
	version=version,
	description='Asynchronous Multiplexing HTTP2/DBO Requests',
	url = 'https://gitlab.com/hansroh/aquests',
	author='Hans Roh',
	author_email='hansroh@gmail.com',	
	packages=packages,
	package_dir=package_dir,
	package_data = package_data,
	license='MIT',
	platforms = ["posix", "nt"],
	download_url = "https://pypi.python.org/pypi/aquests",
	install_requires = install_requires,
	classifiers=classifiers
)
